@@ -184,48 +184,100 @@ impl SessionGroup {
     pub async fn await_stale_sessions(&self, status: &StaleSessionStatus) {
         loop {
             let notified = self.notify.notified();
-            let l = self.sessions.lock().unwrap();
-            let all_gone = l
-                .sessions
-                .iter()
-                .all(|s| !s.maybe_playing || s.seqnum >= status.next_seqnum);
-            drop(l);
-            if all_gone {
-                return;
+            {
+                let l = self.sessions.lock().unwrap();
+                let all_gone = l
+                    .sessions
+                    .iter()
+                    .all(|s| !s.maybe_playing || s.seqnum >= status.next_seqnum);
+                if all_gone {
+                    return;
+                }
             }
             notified.await;
         }
     }
 
+    /// Removes the session with `seqnum`, removing true iff it existed.
+    fn try_remove_seqnum(&self, seqnum: u64) -> bool {
+        let mut l = self.sessions.lock().unwrap();
+        let i = l.sessions.iter().position(|s| s.seqnum == seqnum);
+        match i {
+            Some(i) => {
+                l.sessions.swap_remove(i);
+                drop(l);
+                self.notify.notify_waiters();
+                true
+            }
+            None => false,
+        }
+    }
+
     /// Notes an unexpected RTSP interleaved data message.
     ///
     /// This is assumed to be due to a live555 RTP/AVP/TCP session that belonged
     /// to a since-closed RTSP connection. If there's no known session which
     /// explains this, adds an unknown session with live555's default timeout,
     /// 65 seconds.
-    fn note_stale_live555_data(&self) {
-        let mut lock = self.sessions.lock().unwrap();
-        for s in &lock.sessions {
-            if s.is_tcp {
-                // This session plausibly explains the packet.
-                // (We could go so far as to examine the data packet's SSRC to
-                // see if it matches one associated with this session. But
-                // retrying once per expiration is probably good enough.)
+    fn note_stale_live555_data(self: Arc<Self>, handle: Option<tokio::runtime::Handle>) {
+        let seqnum;
+        let expires = tokio::time::Instant::now() + std::time::Duration::from_secs(65);
+        {
+            let mut lock = self.sessions.lock().unwrap();
+            for s in &lock.sessions {
+                if s.is_tcp {
+                    // This session plausibly explains the packet.
+                    // (We could go so far as to examine the data packet's SSRC to
+                    // see if it matches one associated with this session. But
+                    // retrying once per expiration is probably good enough.)
+                    log::debug!(
+                        "Unexpected RTSP interleaved packet (live555 stale file descriptor \
+                                 bug) plausibly explained by known stale session. Not adding a \
+                                 session entry."
+                    );
+                    return;
+                }
+            }
+            seqnum = lock.next_seqnum;
+            lock.next_seqnum += 1;
+            log::debug!(
+                "Unexpected RTSP interleaved packet, presumed due to live555 stale file \
+                         descriptor bug; tracking expiration in 65 seconds."
+            );
+            lock.sessions.push(StaleSession {
+                seqnum,
+
+                // The caller *might* have a better guess than 65 seconds via a
+                // SETUP response, but it's also possible for
+                // note_stale_live555_data to be called prior to SETUP.
+                expires,
+                teardown_rx: None,
+                is_tcp: true,
+                maybe_playing: true,
+            });
+        }
+
+        // Spawn a task which removes the stale session at its expiration.
+        // We could instead prune expired entries on stale_session() calls and
+        // set a deadline within await_stale_sessions() calls, which might be
+        // a bit more efficient. But this is simpler given that we already are
+        // spawning tasks for stale sessions created from Session's Drop impl.
+        let handle = match handle {
+            Some(h) => h,
+            None => {
+                log::warn!(
+                    "Unable to launch task to clean up stale live555 file descriptor session"
+                );
                 return;
             }
-        }
-        let seqnum = lock.next_seqnum;
-        lock.next_seqnum += 1;
-        lock.sessions.push(StaleSession {
-            seqnum,
-
-            // The caller *might* have a better guess than 65 seconds via a
-            // SETUP response, but it's also possible for
-            // note_stale_live555_data to be called prior to SETUP.
-            expires: tokio::time::Instant::now() + std::time::Duration::from_secs(65),
-            teardown_rx: None,
-            is_tcp: true,
-            maybe_playing: true,
+        };
+        handle.spawn(async move {
+            tokio::time::sleep_until(expires).await;
+            if !self.try_remove_seqnum(seqnum) {
+                log::warn!("Unable to find stale live555 file descriptor session at expiration");
+            } else {
+                log::debug!("Stale live555 file descriptor bug session presumed expired.");
+            }
         });
     }
 }
@@ -798,20 +850,21 @@ impl RtspConnection {
                         } else if let (ResponseMode::Play, Some(m)) =
                             (&mode, self.channels.lookup(d.channel_id()))
                         {
-                            if m.channel_type == ChannelType::Rtcp {
-                                debug!(
-                                    "ignoring interleaved data message on RTCP channel {} while \
+                            debug!(
+                                "ignoring interleaved data message on {:?} channel {} while \
                                      waiting for response to {} CSeq {}",
-                                    d.channel_id(),
-                                    method,
-                                    cseq
-                                );
-                                continue;
-                            }
+                                m.channel_type,
+                                d.channel_id(),
+                                method,
+                                cseq
+                            );
+                            continue;
                         }
 
                         if let Some(session_group) = options.session_group.as_ref() {
-                            session_group.note_stale_live555_data();
+                            session_group.clone().note_stale_live555_data(
+                                tokio::runtime::Handle::try_current().ok(),
+                            );
                         }
 
                         format!(
@@ -1574,6 +1627,7 @@ impl Session<Playing> {
                 conn.inner.ctx(),
                 &pkt_ctx,
                 &mut timeline,
+                inner.runtime_handle.as_ref(),
                 m.stream_i,
                 data.into_body(),
             )?),
@@ -1582,6 +1636,7 @@ impl Session<Playing> {
                     &inner.options,
                     &pkt_ctx,
                     &mut timeline,
+                    inner.runtime_handle.as_ref(),
                     m.stream_i,
                     data.into_body(),
                 ) {
@@ -1636,7 +1691,14 @@ impl Session<Playing> {
                 match r {
                     Ok(()) => {
                         let msg = Bytes::copy_from_slice(buf.filled());
-                        match rtp_handler.rtcp(&inner.options, &pkt_ctx, &mut timeline, i, msg) {
+                        match rtp_handler.rtcp(
+                            &inner.options,
+                            &pkt_ctx,
+                            &mut timeline,
+                            inner.runtime_handle.as_ref(),
+                            i,
+                            msg,
+                        ) {
                             Ok(Some(p)) => return Poll::Ready(Some(Ok(p))),
                             Ok(None) => buf.clear(),
                             Err(description) => {
@@ -1673,6 +1735,7 @@ impl Session<Playing> {
                             &conn_ctx,
                             &pkt_ctx,
                             &mut timeline,
+                            inner.runtime_handle.as_ref(),
                             i,
                             msg,
                         ) {
@@ -1947,6 +2010,8 @@ impl futures::Stream for Demuxed {
 
 #[cfg(test)]
 mod tests {
+    use std::str::FromStr;
+
     use super::*;
     use crate::testutil::response;
 
@@ -1973,6 +2038,19 @@ mod tests {
         (client, server)
     }
 
+    fn init_logging() {
+        let h = mylog::Builder::new()
+            .set_format(
+                ::std::env::var("MOONFIRE_FORMAT")
+                    .map_err(|_| ())
+                    .and_then(|s| mylog::Format::from_str(&s))
+                    .unwrap_or(mylog::Format::Google),
+            )
+            .set_spec(::std::env::var("MOONFIRE_LOG").as_deref().unwrap_or("info"))
+            .build();
+        let _ = h.install();
+    }
+
     /// Receives a request and sends a response, filling in the matching `CSeq`.
     async fn req_response(
         server: &mut crate::tokio::Connection,
@@ -1994,15 +2072,21 @@ mod tests {
             .unwrap();
     }
 
-    /// Tests the happy path of session initialization.
-    #[tokio::test]
+    /// Tests the happy path from initialization to teardown (first attempt succeeds).
+    #[tokio::test(start_paused = true)]
     async fn simple() {
+        init_logging();
         let (conn, mut server) = connect_to_mock().await;
         let url = Url::parse("rtsp://192.168.5.206:554/h264Preview_01_main").unwrap();
+        let group = Arc::new(SessionGroup::default());
 
         // DESCRIBE.
         let (session, _) = tokio::join!(
-            Session::describe_with_conn(conn, SessionOptions::default(), url),
+            Session::describe_with_conn(
+                conn,
+                SessionOptions::default().session_group(group.clone()),
+                url
+            ),
             req_response(
                 &mut server,
                 rtsp_types::Method::Describe,
@@ -2033,50 +2117,219 @@ mod tests {
                 response(include_bytes!("testdata/reolink_play.txt"))
             ),
         );
-        let session = session.unwrap();
-        tokio::pin!(session);
+        let drop_time;
+        {
+            let session = session.unwrap();
+            tokio::pin!(session);
 
-        // Packet.
-        tokio::join!(
-            async {
-                match session.next().await {
-                    Some(Ok(PacketItem::RtpPacket(p))) => {
-                        assert_eq!(p.ssrc, 0xdcc4a0d8);
-                        assert_eq!(p.sequence_number, 0x41d4);
-                        assert_eq!(&p.payload[..], b"hello world");
+            // Packet.
+            tokio::join!(
+                async {
+                    match session.next().await {
+                        Some(Ok(PacketItem::RtpPacket(p))) => {
+                            assert_eq!(p.ssrc, 0xdcc4a0d8);
+                            assert_eq!(p.sequence_number, 0x41d4);
+                            assert_eq!(&p.payload[..], b"hello world");
+                        }
+                        o => panic!("unexpected item: {:#?}", o),
                     }
-                    o => panic!("unexpected item: {:#?}", o),
-                }
-            },
-            async {
-                let pkt = b"\x80\x60\x41\xd4\x00\x00\x00\x00\xdc\xc4\xa0\xd8hello world";
-                server
-                    .send(rtsp_types::Message::Data(rtsp_types::Data::new(
-                        0,
-                        Bytes::from_static(pkt),
-                    )))
-                    .await
-                    .unwrap();
-            },
-        );
+                },
+                async {
+                    let pkt = b"\x80\x60\x41\xd4\x00\x00\x00\x00\xdc\xc4\xa0\xd8hello world";
+                    server
+                        .send(rtsp_types::Message::Data(rtsp_types::Data::new(
+                            0,
+                            Bytes::from_static(pkt),
+                        )))
+                        .await
+                        .unwrap();
+                },
+            );
+            drop_time = tokio::time::Instant::now();
+        };
 
-        // End of stream.
+        // Drop (initiated by exiting the scope above).
+        // This server advertises an ancient version of live555, so Retina
+        // sends a TEARDOWN even with TCP.
+        let stale_sessions = group.stale_sessions();
+        assert_eq!(stale_sessions.num_sessions, 1);
         tokio::join!(
-            async {
-                assert!(session.next().await.is_none());
-            },
-            async {
-                server.close().await.unwrap();
-            },
+            group.await_stale_sessions(&stale_sessions),
+            req_response(
+                &mut server,
+                rtsp_types::Method::Teardown,
+                response(include_bytes!("testdata/reolink_teardown.txt"))
+            ),
+        );
+        assert_eq!(group.stale_sessions().num_sessions, 0);
+
+        // elapsed is not zero because tokio advances the time unnecessarily, grr.
+        // https://github.com/tokio-rs/tokio/issues/3108
+        let elapsed = tokio::time::Instant::now() - drop_time;
+        assert!(
+            elapsed < std::time::Duration::from_secs(60),
+            "elapsed={:?}",
+            elapsed
         );
     }
 
-    /// Tests ignoring a bogus RTCP message while waiting for PLAY response.
-    #[tokio::test]
-    async fn ignore_early_rtcp() {
+    /// As above, but TEARDOWN fails until session expiration.
+    #[tokio::test(start_paused = true)]
+    async fn session_expiration() {
+        init_logging();
         let (conn, mut server) = connect_to_mock().await;
         let url = Url::parse("rtsp://192.168.5.206:554/h264Preview_01_main").unwrap();
-        let bogus_pkt = rtsp_types::Message::Data(rtsp_types::Data::new(
+        let group = Arc::new(SessionGroup::default());
+
+        // DESCRIBE.
+        let (session, _) = tokio::join!(
+            Session::describe_with_conn(
+                conn,
+                SessionOptions::default().session_group(group.clone()),
+                url
+            ),
+            req_response(
+                &mut server,
+                rtsp_types::Method::Describe,
+                response(include_bytes!("testdata/reolink_describe.txt"))
+            ),
+        );
+        let mut session = session.unwrap();
+        assert_eq!(session.streams().len(), 2);
+
+        // SETUP.
+        tokio::join!(
+            async {
+                session.setup(0).await.unwrap();
+            },
+            req_response(
+                &mut server,
+                rtsp_types::Method::Setup,
+                response(include_bytes!("testdata/reolink_setup.txt"))
+            ),
+        );
+
+        // PLAY.
+        let (session, _) = tokio::join!(
+            session.play(PlayOptions::default()),
+            req_response(
+                &mut server,
+                rtsp_types::Method::Play,
+                response(include_bytes!("testdata/reolink_play.txt"))
+            ),
+        );
+        let drop_time;
+        {
+            let session = session.unwrap();
+            tokio::pin!(session);
+
+            // Packet.
+            tokio::join!(
+                async {
+                    match session.next().await {
+                        Some(Ok(PacketItem::RtpPacket(p))) => {
+                            assert_eq!(p.ssrc, 0xdcc4a0d8);
+                            assert_eq!(p.sequence_number, 0x41d4);
+                            assert_eq!(&p.payload[..], b"hello world");
+                        }
+                        o => panic!("unexpected item: {:#?}", o),
+                    }
+                },
+                async {
+                    let pkt = b"\x80\x60\x41\xd4\x00\x00\x00\x00\xdc\xc4\xa0\xd8hello world";
+                    server
+                        .send(rtsp_types::Message::Data(rtsp_types::Data::new(
+                            0,
+                            Bytes::from_static(pkt),
+                        )))
+                        .await
+                        .unwrap();
+                },
+            );
+            drop_time = tokio::time::Instant::now();
+        }
+
+        // Drop (initiated by exiting the scope above).
+        // This server advertises an ancient version of live555, so Retina
+        // sends a TEARDOWN even with TCP.
+        server.close().await.unwrap();
+        let stale_sessions = group.stale_sessions();
+        assert_eq!(stale_sessions.num_sessions, 1);
+
+        // Even if repeated attempts fail, the stale session will go await on timeout.
+        // The "60" here is the RFC-defined default timeout when none is specified
+        // in the SETUP response.
+        group.await_stale_sessions(&stale_sessions).await;
+        assert_eq!(group.stale_sessions().num_sessions, 0);
+
+        // elapsed is not zero because tokio advances the time unnecessarily, grr.
+        // https://github.com/tokio-rs/tokio/issues/3108
+        let elapsed = tokio::time::Instant::now() - drop_time;
+        assert!(
+            elapsed >= std::time::Duration::from_secs(60),
+            "elapsed={:?}",
+            elapsed
+        );
+    }
+
+    /// Stale sessions detected via unexpected RTSP interleaved packets should be tracked
+    /// until expiration.
+    #[tokio::test(start_paused = true)]
+    async fn stale_file_descriptor_session() {
+        init_logging();
+        log::debug!("debug logging works");
+        let (conn, mut server) = connect_to_mock().await;
+        let url = Url::parse("rtsp://192.168.5.206:554/h264Preview_01_main").unwrap();
+        let group = Arc::new(SessionGroup::default());
+        let bogus_rtp = rtsp_types::Message::Data(rtsp_types::Data::new(
+            0,                                // RTP channel
+            Bytes::from_static(b"bogus pkt"), // the real packet parses but this is fine.
+        ));
+
+        let start = tokio::time::Instant::now();
+
+        // DESCRIBE.
+        tokio::join!(
+            async {
+                let e = Session::describe_with_conn(
+                    conn,
+                    SessionOptions::default().session_group(group.clone()),
+                    url,
+                )
+                .await
+                .map(|_s| ())
+                .unwrap_err();
+                assert!(matches!(*e.0, ErrorInt::RtspFramingError { .. }));
+            },
+            async { server.send(bogus_rtp).await.unwrap() },
+        );
+
+        let stale_sessions = group.stale_sessions();
+        assert_eq!(stale_sessions.num_sessions, 1);
+
+        group.await_stale_sessions(&stale_sessions).await;
+        let elapsed = tokio::time::Instant::now() - start;
+        assert_eq!(group.stale_sessions().num_sessions, 0);
+
+        // 65 seconds is the hardcoded time for a live555 stale TCP session.
+        assert!(
+            elapsed >= std::time::Duration::from_secs(65),
+            "elapsed={:?}",
+            elapsed
+        );
+    }
+
+    /// Tests ignoring bogus RTP and RTCP messages while waiting for PLAY response.
+    #[tokio::test]
+    async fn ignore_early_rtp_rtcp() {
+        init_logging();
+        let (conn, mut server) = connect_to_mock().await;
+        let url = Url::parse("rtsp://192.168.5.206:554/h264Preview_01_main").unwrap();
+        let bogus_rtp = rtsp_types::Message::Data(rtsp_types::Data::new(
+            0,                                // RTP channel
+            Bytes::from_static(b"bogus pkt"), // the real packet parses but this is fine.
+        ));
+        let bogus_rtcp = rtsp_types::Message::Data(rtsp_types::Data::new(
             1,                                // RTCP channel
             Bytes::from_static(b"bogus pkt"), // the real packet parses but this is fine.
         ));
@@ -2110,7 +2363,8 @@ mod tests {
 
         // PLAY.
         let (session, _) = tokio::join!(session.play(PlayOptions::default()), async move {
-            server.send(bogus_pkt).await.unwrap();
+            server.send(bogus_rtp).await.unwrap();
+            server.send(bogus_rtcp).await.unwrap();
             req_response(
                 &mut server,
                 rtsp_types::Method::Play,
@@ -2124,6 +2378,7 @@ mod tests {
     // See with: cargo test -- --nocapture client::tests::print_sizes
     #[test]
     fn print_sizes() {
+        init_logging();
         for (name, size) in &[
             ("PacketItem", std::mem::size_of::<PacketItem>()),
             ("Presentation", std::mem::size_of::<Presentation>()),
@@ -2148,10 +2403,20 @@ mod tests {
 
     #[test]
     fn check_live555_tcp_bug() {
+        init_logging();
         assert!(!has_live555_tcp_bug("not live555"));
         assert!(!has_live555_tcp_bug("LIVE555 Streaming Media v"));
         assert!(has_live555_tcp_bug("LIVE555 Streaming Media v2013.04.08"));
         assert!(!has_live555_tcp_bug("LIVE555 Streaming Media v2017.06.04"));
         assert!(!has_live555_tcp_bug("LIVE555 Streaming Media v2020.01.01"));
     }
+
+    #[test]
+    fn await_stale_sessions_is_send() {
+        // There's probably a more elegant way to test this, but here goes.
+        fn assert_send<T: Send>(_: T) {}
+        let group = SessionGroup::default();
+        let stale_sessions = group.stale_sessions();
+        assert_send(group.await_stale_sessions(&stale_sessions));
+    }
 }
