@@ -24,7 +24,11 @@ pub(super) async fn background_teardown(
     mut tx: tokio::sync::watch::Sender<Option<Result<(), Error>>>,
     expires: tokio::time::Instant,
 ) {
-    log::debug!("TEARDOWN {} starting", &*session_id);
+    log::debug!(
+        "TEARDOWN {} starting for URL {}",
+        &*session_id,
+        base_url.as_str()
+    );
     if tokio::time::timeout_at(
         expires,
         teardown_loop_forever(
@@ -43,15 +47,8 @@ pub(super) async fn background_teardown(
     }
     if let Some(ref session_group) = options.session_group {
         let seqnum = seqnum.expect("seqnum specified when session_group exists");
-        let mut l = session_group.sessions.lock().unwrap();
-        let i = l.sessions.iter().position(|s| s.seqnum == seqnum);
-        match i {
-            Some(i) => {
-                l.sessions.swap_remove(i);
-                drop(l);
-                session_group.notify.notify_waiters();
-            }
-            None => log::warn!("Unable to find session {:?} on TEARDOWN", &*session_id),
+        if !session_group.try_remove_seqnum(seqnum) {
+            log::warn!("Unable to find session {:?} on TEARDOWN", &*session_id);
         }
     }
 
